
/**
 *  The class BSTree implements a binary tree with the following
 * property: for each node n, every left-subtree value is less then the key value
 * v stored in n and every right-subtree value is greater then v.
 *
 */


public class BSTree {
    protected BSTNode root = null;
    protected int size = 0;

    /**
     * Creates an empty BSTree
     */
    public BSTree(){
    }


    /**
     * Return the BSTree root node
     * @return the BSTNode that contains the tree root.
     */
    public BSTNode root(){
      return root;
    }


    /**
     * Method to get element field.
     * @param v the node.
     * @return the element field or null if t is null.
     */
    public Comparable element( BSTNode v )
        {
            return v == null ? null : v.key;
        }


        /**
     * Verifies if the node v is a leaf.
     * @param v the node
     * @return true if v is a leaf, false otherwise.
     */
    public boolean isLeaf(BSTNode v){
      return (v.getLeftChild()==null && v.getRightChild()==null);
    }


    /**
     * Verifies if the node v is the root of this tree
     * @param v the node
     * @return true if the node v is root, false otherwise.
     */
    public boolean isRoot(BSTNode v){
      return (this.root==v);
    }


    /**
     * Inserts the specified element in this BSTree.
     * @param el element to insert.
     * @return the object that represents the inserted node.
     */


    public Object insert(Comparable el){
      return (Object)insert(new BSTNode(el,null,null));
    }


    /**
     * Removes the element el from this tree
     * @param el the elment to remove
     * @return the element el if it was in the tree, null otherwise.
     */
    public Comparable remove(Comparable el) {
            BSTNode p = root;
            BSTNode parent = null;
            while(p != null)
                    if(el.compareTo(p.key) > 0) {
                                   parent = p;
                            p = p.rightChild;
                    } else if(el.compareTo(p.key) < 0) {
                            parent = p;
                            p = p.leftChild;
                    } else break;
            if(p == null) return null; // el not in tree

            // 1st case: p is leaf
            if(p.isLeaf())
                    if(parent == null) // p is root, too
                            root = null;
                    else if(parent.rightChild == p) // p is rightChild
                            parent.rightChild = null;
                    else	// p is leftChild
                            parent.leftChild = null;

            // 2nd case: p has one child
            else if(p.leftChild == null)	// p has only a rightChild
                    if(parent == null) // p is root, too
                            root = p.rightChild;
                           else if(parent.rightChild == p)	// p is rightChild
                                   parent.rightChild = p.rightChild;
                           else	// p is leftChild
                                   parent.leftChild = p.rightChild;
                   else if(p.rightChild == null)	// p has only a leftChild
                           if(parent == null) // p is root, too
                                   root = p.leftChild;
                         else if(parent.rightChild == p)	// p is rightChild
                                parent.rightChild = p.leftChild;
                        else	// p is leftChild
                                parent.leftChild = p.leftChild;

                // 3rd case: p has 2 children (then it has a predecessor!)
                else {Comparable pred = this.predecessor(p.leftChild, p.key).key;
                        this.remove(pred); // removing node doesn't have two children!
                        p.key = pred;
                }
                size--;
                return el;
    }




    /**
     * Finds the specified element el.
     * @param el the element to search.
     * @return the node that contains the specified element el.
     */
    public Object find(Comparable el){
      return (Object)search(el);
    }





/******************************************************************************/
/*                               methods                                 */
/******************************************************************************/

    /**
     * Returns the left child node of v
     * @param v the parent node
     * @return the left child node
     */
    public BSTNode leftChild(BSTNode v){
      return v.getLeftChild();

    }




    /**
     * Returns the right child node of v
     * @param v the parent node
     * @return the right child node
     */
    public BSTNode rightChild(BSTNode v){
      return v.getRightChild();
    }



    /**
     * Removes all of the elements from this BSTree
     */
    public void clear() {
        root = null;
        size = 0;
    }


    /**
     * Tests if this BSTree has no components.
     * @return true if and only if this BSTree has no components, that is,
     * its size is zero; false otherwise
     */
    public boolean isEmpty() {
        return root == null;
    }


    /**
     * Searches for the element el in this BSTree.
     * @param el the element to search for.
     * @return the node that contains el, if any exists; null otherwise.
     */
    public BSTNode search(Comparable el) {
        return this.search(root,el);
    }


    /**
     * Recorsive method to find the element el, starting from the specified node p
     * @param p the node to start searching from.
     * @param el the element.
     * @return the node that contains el, if any exists; null otherwise.
     */
    public BSTNode search(BSTNode p, Comparable el) {
            if(p == null)
                    return null;
            if(el.compareTo(p.key) == 0)
                    return p;
            if (el.compareTo(p.key) < 0)
                    return this.search(p.leftChild, el);
            else
                    return this.search(p.rightChild, el);
    }



    /**
     * Inserts the specified node in this BSTree.
     * @param node node to insert.
     * @return the inserted node.
     */
    public BSTNode insert(BSTNode node) {
        BSTNode p = root, prev = null;
        Comparable el = node.getKey();
        while (p != null) {  // find a place for inserting new node;
            prev = p;
            if (p.key.compareTo(el) < 0)
                    p = p.rightChild;
            else if (p.key.compareTo(el) > 0)
                    p = p.leftChild;
            else return null; // duplicate element;
        }
        if (root == null)    // tree is empty;
             root = node;
        else if (prev.key.compareTo(el) < 0)
             prev.rightChild = node;
        else prev.leftChild  = node;
        this.size++;
        return node;
    }





    /**
     * Tests if the specified object is in this BSTree.
     * @param el the object to test.
     * @return true if and only if the specified object is in this BSTree; false
     * otherwise.
     */
    public boolean isInTree(Comparable el) {
        return search(el) != null;
    }


    /**
     * Returns the number of nodes in this BSTree.
     * @return the number of nodes in this BSTree.
     */
    public int getSize() {
        return this.size;
    }


    /**
     * Implements a preorder traversal of this BSTree, beginning from the root.
     * Calls the recorsive method preorder(BSTNode p)
     */
    public void preorder() {
        preorder(root);
    }


    /**
     * Implements a inorder traversal of this BSTree, beginning from the root.
     * Calls the recorsive method inorder(BSTNode p).
     */
    public void inorder() {
        inorder(root);
    }


    /**
     * Implements a postorder traversal of this BSTree, beginning from the root.
     * Calls the recorsive method postorder(BSTNode p)
     */
    public void postorder() {
        postorder(root);
    }


    /**
     * Recorsive method to implement the inorder traversal, beginning from the
     * specified node p.
     * @param p a node.
     */
    public void inorder(BSTNode p) {
        if (p != null) {
             inorder(p.leftChild);
             p.visit();
             inorder(p.rightChild);
        }
    }


    /**
     * Recorsive method to implement the preorder traversal, beginning from the
     * specified node p.
     * @param p a node.
     */
    public void preorder(BSTNode p) {
        if (p != null) {
             p.visit();
             preorder(p.leftChild);
             preorder(p.rightChild);
        }
    }


    /**
     * Recorsive method to implement the postorder traversal, beginning from the
     * specified node p.
     * @param p a node.
     */
    public void postorder(BSTNode p) {
        if (p != null) {
             postorder(p.leftChild);
             postorder(p.rightChild);
             p.visit();
        }
    }



    /**
     * Returns the height of this BSTree. Calls the recorsive method treeHeight (BSTNode node).
     * @return the height of this BSTree
     */
    public int treeHeight() {
            return treeHeight(this.root);
    }


    /**
     * Returns the height of the subtree with root in node.
     * @param node a node.
     * @return the height of the subtree with the specified node as root.
     */
    public int treeHeight(BSTNode node) {
            if(node == null)
                    return 0;
            if(node.isLeaf())
                    return 1;
            else {
                    int height;
                    int lheight = treeHeight(node.leftChild);
                    int rheight = treeHeight(node.rightChild);
                    height = lheight > rheight ? lheight : rheight;
                    return 1 + height;
            }
    }

    /**
     * Searches the predecessor of the node with key equals to el.
     * @param el the specified element.
     * @return the predecessor node.
     */
    public BSTNode predecessor(Comparable el) {
        return this.predecessor(root, el);
      }


      /**
       * Searches the predecessor of the node with key equals to el, starting
       * from the node p
       * @param p the specified node
       * @param el key of the specified node.
       * @return the predecessor node of el.
       */
      public BSTNode predecessor(BSTNode p, Comparable el) {
      BSTNode pred = null;
      while(p != null)
        if(el.compareTo(p.key) <= 0)
          p = p.leftChild;
        else {
          pred = p;
          p = p.rightChild;
        }
        return pred;
      }


      /**
       * Searches the successor of the node with key equals to el.
       * @param el the specified element.
       * @return the successor node.
       */
      public BSTNode successor(Comparable el) {
        return this.successor(root, el);
      }


      /**
       * Searches the successor of the node with key equals to el, starting
       * from the node p
       * @param p the specified node
       * @param el key of the specified node.
       * @return the successor node of el.
       */
      public BSTNode successor(BSTNode p, Comparable el) {
        BSTNode succ = null;
        while(p != null)
                if(el.compareTo(p.key) >= 0)
                        p = p.rightChild;
                else {
                        succ = p;
                        p = p.leftChild;
                }
        return succ;
      }


    /**
     * Inserts the element from the specified array into this tree.
     * @param data array of elements.
     * @param first first index in the array.
     * @param last index in the array.
     */
    public void balance(Comparable data[], int first, int last) {
        if (first <= last) {
            int middle = (first + last)/2;
            insert(data[middle]);
            balance(data,first,middle-1);
            balance(data,middle+1,last);
        }
    }


}